package com.sun.org.apache.xerces.internal.impl.xpath.regex;

import java.text.CharacterIterator;

public class BMPattern
{
    final char[] pattern;
    final int[] shiftTable;
    final boolean ignoreCase;
    
    public BMPattern(String pat, boolean ignoreCase) {
        this(pat, 256, ignoreCase);
    }
    
    public BMPattern(String pat, int tableSize, boolean ignoreCase) {
        this.pattern = pat.toCharArray();
        this.shiftTable = new int[tableSize];
        this.ignoreCase = ignoreCase;
        int length = this.pattern.length;
        for (int i = 0; i < this.shiftTable.length; ++i) {
            this.shiftTable[i] = length;
        }
        for (int i = 0; i < length; ++i) {
            char ch = this.pattern[i];
            int diff = length - i - 1;
            int index = ch % this.shiftTable.length;
            if (diff < this.shiftTable[index]) {
                this.shiftTable[index] = diff;
            }
            if (this.ignoreCase) {
                ch = Character.toUpperCase(ch);
                index = ch % this.shiftTable.length;
                if (diff < this.shiftTable[index]) {
                    this.shiftTable[index] = diff;
                }
                ch = Character.toLowerCase(ch);
                index = ch % this.shiftTable.length;
                if (diff < this.shiftTable[index]) {
                    this.shiftTable[index] = diff;
                }
            }
        }
    }
    
    public int matches(CharacterIterator iterator, int start, int limit) {
        if (this.ignoreCase) {
            return this.matchesIgnoreCase(iterator, start, limit);
        }
        int plength = this.pattern.length;
        if (plength == 0) {
            return start;
        }
        int index = start + plength;
    Label_0035:
        while (index <= limit) {
            int pindex = plength;
            int nindex = index + 1;
            Label_0094: {
                char ch;
                while ((ch = iterator.setIndex(--index)) == this.pattern[--pindex]) {
                    if (pindex == 0) {
                        return index;
                    }
                    if (pindex > 0) {
                        continue;
                    }
                    index += this.shiftTable[ch % this.shiftTable.length] + 1;
                    if (index < nindex) {
                        index = nindex;
                    }
                    continue Label_0035;
                }
                continue;
            }
        }
        return -1;
    }
    
    public int matches(String str, int start, int limit) {
        if (this.ignoreCase) {
            return this.matchesIgnoreCase(str, start, limit);
        }
        int plength = this.pattern.length;
        if (plength == 0) {
            return start;
        }
        int index = start + plength;
    Label_0035:
        while (index <= limit) {
            int pindex = plength;
            int nindex = index + 1;
            Label_0092: {
                char ch;
                while ((ch = str.charAt(--index)) == this.pattern[--pindex]) {
                    if (pindex == 0) {
                        return index;
                    }
                    if (pindex > 0) {
                        continue;
                    }
                    index += this.shiftTable[ch % this.shiftTable.length] + 1;
                    if (index < nindex) {
                        index = nindex;
                    }
                    continue Label_0035;
                }
                continue;
            }
        }
        return -1;
    }
    
    public int matches(char[] chars, int start, int limit) {
        if (this.ignoreCase) {
            return this.matchesIgnoreCase(chars, start, limit);
        }
        int plength = this.pattern.length;
        if (plength == 0) {
            return start;
        }
        int index = start + plength;
    Label_0035:
        while (index <= limit) {
            int pindex = plength;
            int nindex = index + 1;
            Label_0090: {
                char ch;
                while ((ch = chars[--index]) == this.pattern[--pindex]) {
                    if (pindex == 0) {
                        return index;
                    }
                    if (pindex > 0) {
                        continue;
                    }
                    index += this.shiftTable[ch % this.shiftTable.length] + 1;
                    if (index < nindex) {
                        index = nindex;
                    }
                    continue Label_0035;
                }
                continue;
            }
        }
        return -1;
    }
    
    int matchesIgnoreCase(CharacterIterator iterator, int start, int limit) {
        int plength = this.pattern.length;
        if (plength == 0) {
            return start;
        }
        int index = start + plength;
        while (index <= limit) {
            int pindex = plength;
            int nindex = index + 1;
            char ch;
            do {
                char ch2;
                ch = (ch2 = iterator.setIndex(--index));
                char ch3 = this.pattern[--pindex];
                if (ch2 != ch3) {
                    ch2 = Character.toUpperCase(ch2);
                    ch3 = Character.toUpperCase(ch3);
                    if (ch2 != ch3 && Character.toLowerCase(ch2) != Character.toLowerCase(ch3)) {
                        break;
                    }
                }
                if (pindex == 0) {
                    return index;
                }
                continue;
            } while (pindex > 0);
            index += this.shiftTable[ch % this.shiftTable.length] + 1;
            if (index < nindex) {
                index = nindex;
            }
            else {
                continue;
            }
        }
        return -1;
    }
    
    int matchesIgnoreCase(String text, int start, int limit) {
        int plength = this.pattern.length;
        if (plength == 0) {
            return start;
        }
        int index = start + plength;
        while (index <= limit) {
            int pindex = plength;
            int nindex = index + 1;
            char ch;
            do {
                char ch2;
                ch = (ch2 = text.charAt(--index));
                char ch3 = this.pattern[--pindex];
                if (ch2 != ch3) {
                    ch2 = Character.toUpperCase(ch2);
                    ch3 = Character.toUpperCase(ch3);
                    if (ch2 != ch3 && Character.toLowerCase(ch2) != Character.toLowerCase(ch3)) {
                        break;
                    }
                }
                if (pindex == 0) {
                    return index;
                }
                continue;
            } while (pindex > 0);
            index += this.shiftTable[ch % this.shiftTable.length] + 1;
            if (index < nindex) {
                index = nindex;
            }
            else {
                continue;
            }
        }
        return -1;
    }
    
    int matchesIgnoreCase(char[] chars, int start, int limit) {
        int plength = this.pattern.length;
        if (plength == 0) {
            return start;
        }
        int index = start + plength;
        while (index <= limit) {
            int pindex = plength;
            int nindex = index + 1;
            char ch;
            do {
                char ch2;
                ch = (ch2 = chars[--index]);
                char ch3 = this.pattern[--pindex];
                if (ch2 != ch3) {
                    ch2 = Character.toUpperCase(ch2);
                    ch3 = Character.toUpperCase(ch3);
                    if (ch2 != ch3 && Character.toLowerCase(ch2) != Character.toLowerCase(ch3)) {
                        break;
                    }
                }
                if (pindex == 0) {
                    return index;
                }
                continue;
            } while (pindex > 0);
            index += this.shiftTable[ch % this.shiftTable.length] + 1;
            if (index < nindex) {
                index = nindex;
            }
            else {
                continue;
            }
        }
        return -1;
    }
}
